@@ -1,6 +1,6 @@
 #
 #
-# (C) Copyright 2011 Enthought, Inc., Austin, TX
+# (C) Copyright 2011-2022 Enthought, Inc., Austin, TX
 # All right reserved.
 #
 # This file is open source software distributed according to the terms in LICENSE.txt
@@ -16,8 +16,8 @@ import itertools
 import bisect
 import heapq
 import threading
-from types import MethodType
 import weakref
+from types import MethodType
 import traceback
 
 # Logging.
@@ -32,7 +32,7 @@ from .abstract_event_manager import BaseEvent, BaseEventManager
 ###############################################################################
 class LoggingTracer(object):
     """ A tracer object for event manager to log events.
-    
+
     Usage
     -----
     event_manager.set_trace(LoggingTracer())
@@ -67,15 +67,10 @@ class MethodNotifier(object):
     """
     __slots__ = ['func', 'cls', 'obj', '_notify', '_args']
     def __init__(self, meth, notify=None, args=()):
-        self.func = meth.im_func
-        self.cls = meth.im_class
-        obj = meth.im_self
-        if obj is None:
-            # Unbound Method.
-            self.obj = None
-        else:
-            # Bound method.
-            self.obj = weakref.ref(meth.im_self, self.notify)
+        self.func = meth.__func__
+
+        self.cls = meth.__self__.__class__
+        self.obj = weakref.ref(meth.__self__, self.notify)
         if notify:
             self._notify = notify
             self._args = args
@@ -89,15 +84,12 @@ class MethodNotifier(object):
         """ Return the original listener method, or None if it no longer exists.
         """
         obj = self.obj
-        if obj is None:
-            # Unbound method.
-            objc = None
-        else:
-            objc = obj()
-            if objc is None:
-                # Bound method whose object has been garbage collected.
-                return
-        return MethodType(self.func, objc, self.cls)
+        objc = obj()
+        if objc is None:
+            # Bound method whose object has been garbage collected.
+            return
+
+        return MethodType(self.func, objc)
 
 ###############################################################################
 # `EventInfo` Private Class.
@@ -140,7 +132,7 @@ class EventInfo(object):
         filter : dict
             Filters to match for before calling the listener. The listener is
             called only when the event matches all of the filter .
-            
+
             Filter specification:
                 - key: string which is extended name of an attribute of the
                     event instance. For example string 'source.name' will be
@@ -158,7 +150,7 @@ class EventInfo(object):
 
         Note
         ----
-       
+
         Reconnecting an already connected listener will disconnect the
         old listener. This may have rammifications in changing the filters
         and the priority.
@@ -211,13 +203,13 @@ class EventInfo(object):
     def get_id(self, func):
         """ Get an id as unique key for the function. """
         if type(func) is MethodType:
-            obj = func.im_self
+            obj = func.__self__
             if obj is None:
                 # Unbound method
-                return weakref.ref(func.im_func),weakref.ref(func.im_class)
+                return weakref.ref(func.__func__),weakref.ref(func.__self__.__class__)
             else:
                 # Bound method.
-                return weakref.ref(func.im_func),weakref.ref(func.im_self)
+                return weakref.ref(func.__func__),weakref.ref(func.__self__)
         else:
             return func
 
@@ -250,7 +242,7 @@ class EventInfo(object):
                 listener = linfo[-1]
                 id = self.get_id(listener())
                 if id in l_filter:
-                    for key, value in l_filter[id].iteritems():
+                    for key, value in l_filter[id].items():
                         attr = event
                         try:
                             # Get extended attributes of the event.
@@ -334,7 +326,7 @@ class EventManager(BaseEventManager):
         filter : dict
             Filters to match for before calling the listener. The listener is
             called only when the event matches all of the filter .
-            
+
             Filter specification:
                 - key: string which is name of an attribute of the event instance.
                 - value: the value of the specified attribute.
@@ -345,7 +337,7 @@ class EventManager(BaseEventManager):
 
         Note
         ----
-       
+
         Reconnecting an already connected listener will disconnect the
         old listener. This may have rammifications in changing the filters
         and the priority.
@@ -378,7 +370,7 @@ class EventManager(BaseEventManager):
         ------
         KeyError :
             if `func` is not already connected.
-            
+
         """
         if self._trace_func is not None:
             if self._trace_func('disconnect', self.disconnect,
@@ -401,7 +393,7 @@ class EventManager(BaseEventManager):
 
         Note
         ----
-        
+
         Listeners of superclasses of the event are also called.
         Eg. a :py:class:`BaseEvent` listener will also be notified about any
         derived class events.
@@ -437,8 +429,10 @@ class EventManager(BaseEventManager):
                     'event: {2}:\n{3}'.format(e, listener, event,
                                               traceback.format_exc()))
             if event._handled:
-                logger.info('Event: {0} handled by listener: {1}'.format(
-                                                        event, listener))
+                # Only enable when debugging -- very slow even if it doesn't get emitted because
+                # it must still do the string formatting.
+                #logger.debug('Event: {0} handled by listener: {1}'.format(
+                #                                        event, listener))
                 break
 
         event.post_emit()
@@ -449,15 +443,15 @@ class EventManager(BaseEventManager):
         Parameters
         ----------
         cls : class
-            The class of the event we want the ``EventInfo`` for.        
+            The class of the event we want the ``EventInfo`` for.
             If ``cls`` is ``None``, then all known event types are returned.
-        
+
         Returns
         -------
         event_info :
             An ``EventInfo`` instance for the class, or a dictionary mapping
             classes to ``EventInfo`` instances.
-        
+
         """
         if cls is None:
             return self.event_map
@@ -466,25 +460,25 @@ class EventManager(BaseEventManager):
 
     def get_listeners(self, event, cls=None):
         """ Return listeners which will be called on specified event.
-        
+
         Parameters
         ----------
-        
+
         event : event instance or class
             The event we want to get the listeners for.
             If ``event`` is an instance of BaseEvent(), the listeners which will
             be called for the event are returned (satisfying any filters on the
             listeners).
-            
+
             If ``event`` is BaseEvent subclass, all listeners for specified event
             class are returned (but no filtering is performed).
-        
+
         cls : BaseEvent subclass
             ``cls`` argument is generally not needed, it is for internal use.
             If ``cls`` is specified as a subclass of ``BaseEvent``, then only
             listeners for the specified event class and superclasses are
             returned.
-        
+
         """
         evt_map = self.event_map
         if cls is None:
@@ -541,7 +535,7 @@ class EventManager(BaseEventManager):
 
     def get_event_hierarchy(self, cls):
         """ The the sequence of event classes which are notified for given cls.
-        
+
         """
         return cls.__mro__[:self.bmro_clip]
 
