@@ -1,5 +1,5 @@
 #
-# (C) Copyright 2012 Enthought, Inc., Austin, TX
+# (C) Copyright 2011-2022 Enthought, Inc., Austin, TX
 # All right reserved.
 #
 # This file is open source software distributed according to the terms in LICENSE.txt
@@ -27,18 +27,19 @@ Ex. Client A reads file1 and then reads file2, whereas client B reads file2
 """
 
 # System library imports.
-import os
-import glob
 from contextlib import contextmanager
-from functools import wraps
 import datetime
+from functools import wraps
+import glob
+import io
+import os
 import time
 import threading
 
 # ETS library imports.
 from .events import StoreSetEvent, StoreUpdateEvent,\
     StoreDeleteEvent, StoreKeyEvent
-from .filesystem_store import FileSystemStore, init_shared_store
+from .filesystem_store import FileSystemStore
 from .file_lock import FileLock
 from .utils import SimpleTransactionContext
 
@@ -47,7 +48,7 @@ def transact(function, on_commit=True):
     """ Wrap a store method to add command to transaction instead of executing
     it immediately if within a transaction context. If not in a transaction
     context, the command is executed immediately.
-    
+
     Parameters
     ----------
     function - callable
@@ -127,12 +128,13 @@ def write_log(func, event='w'):
                 # FIXME: Assuming max line length is 1024
                 log_file.seek(max(0, end_pos-1024))
                 etext = log_file.read()
+                log_file.seek(0) # Needed on windows to be able to write.
                 text = etext.splitlines()[-1]
                 id = int(text.split()[0]) + 1
             else:
                 id = 1
-            new_line = '%d %s %s %s\n'%(id, event, time, key)
-            log_file.write(new_line)
+            new_line = '%d %s %s %s\n' % (id, event, time, key)
+            log_file.write(new_line.encode('utf-8'))
             log_file.close()
 
             if id % store._logrotate_limit == 0:
@@ -149,16 +151,16 @@ class LockingFileSystemStore(FileSystemStore):
     # Basic Create/Read/Update/Delete Methods
     ##########################################################################
 
-    def __init__(self, event_manager, path, force_lock_timeout=10.0,
+    def __init__(self, path, force_lock_timeout=10.0,
                  magic_fname='.FSStore',
                  remote_event_poll_interval=5.0,
                  max_time_delta=datetime.timedelta(minutes=1)):
-        """Initializes the store given a path to a store. 
-        
+        """Initializes the store given a path to a store.
+
         Parameters
-        ----------        
+        ----------
         event_manager : Event Manager instance
-            This is used to emit suitable events.        
+            This is used to emit suitable events.
         path : str:
             A path to the root of the file system store.
         force_lock_timeout: float
@@ -173,10 +175,10 @@ class LockingFileSystemStore(FileSystemStore):
             The maximum permissible timedelta between different clients.
             This value is used to return the keys in a query_key when the
             parameters are ``last_modified__gte=timedelta`` , in this case the
-            max_time_delta is subtracted from the given query timedelta. 
+            max_time_delta is subtracted from the given query timedelta.
 
         """
-        super(LockingFileSystemStore, self).__init__(event_manager, path, magic_fname)
+        super(LockingFileSystemStore, self).__init__(path, magic_fname)
         self._force_lock_timeout = force_lock_timeout
         self._transaction = None
         self._transaction_locks = []
@@ -193,7 +195,7 @@ class LockingFileSystemStore(FileSystemStore):
         try:
             # Create the log file atomically.
             os.close(os.open(self._log_file, os.O_CREAT|os.O_EXCL|os.O_RDWR))
-        except OSError as e:
+        except OSError:
             pass
 
         self._remote_event_poll_interval = remote_event_poll_interval
@@ -227,17 +229,17 @@ class LockingFileSystemStore(FileSystemStore):
 
     def query(self, select=None, **kwargs):
         """ Query for keys and metadata matching metadata provided as keyword arguments
-        
+
         This provides a very simple querying interface that returns precise
         matches with the metadata.  If no arguments are supplied, the query
         will return the complete set of metadata for the key-value store.
-        
+
         Parameters
         ----------
         select : iterable of strings or None
             An optional list of metadata keys to return.  If this is not None,
             then the metadata dictionaries will only have values for the specified
-            keys populated.        
+            keys populated.
         kwargs :
             Arguments where the keywords are metadata keys, and values are
             possible values for that metadata item.
@@ -249,7 +251,7 @@ class LockingFileSystemStore(FileSystemStore):
             all the specified values for the specified metadata keywords.
             If a key specified in select is not present in the metadata of a
             particular key, then it will not be present in the returned value.
-        
+
         """
         all_metadata = glob.glob(os.path.join(self._root, '*.metadata'))
         items = [(os.path.splitext(os.path.basename(x))[0], x) for x in all_metadata]
@@ -266,17 +268,17 @@ class LockingFileSystemStore(FileSystemStore):
                 metadata = self._get_metadata(path)
                 if all(metadata.get(arg) == value for arg, value in kwargs.items()):
                     yield key, metadata.copy()
-    
+
     def query_keys(self, **kwargs):
         """ Query for keys matching metadata provided as keyword arguments
-        
+
         This provides a very simple querying interface that returns precise
         matches with the metadata.  If no arguments are supplied, the query
         will return the complete set of keys for the key-value store.
-        
+
         This is equivalent to ``self.query(**kwargs).keys()``, but potentially
         more efficiently implemented.
-        
+
         Parameters
         ----------
         kwargs :
@@ -288,13 +290,13 @@ class LockingFileSystemStore(FileSystemStore):
         result : iterable
             An iterable of key-value store keys whose metadata matches all the
             specified values for the specified metadata keywords.
-        
+
         """
         # Optimize for special cases.
         basename = os.path.basename
-        if kwargs.keys() == ['type']:
-            typ =  kwargs['type']
-            if typ in ('file','dir'):
+        if 'type' in kwargs:
+            typ = kwargs['type']
+            if typ in ('file', 'dir'):
                 pattern = '{0}.*.metadata'.format(typ)
                 for i in glob.iglob(os.path.join(self._root, pattern)):
                     yield basename(i)[:-9]
@@ -320,14 +322,14 @@ class LockingFileSystemStore(FileSystemStore):
 
         """
         timestamp = since
-        if isinstance(timestamp, basestring):
+        if isinstance(timestamp, str):
             ISO_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
             timestamp = datetime.datetime.strptime(timestamp, ISO_FORMAT)
         timestamp -= self._max_time_delta
         write = False
         modified_keys = []
-        for id,typ,time,key in self._log_iter():
-            if write is False and time > timestamp:
+        for id, typ, mtime, key in self._log_iter():
+            if write is False and mtime > timestamp:
                 write = True
             if write is True:
                 modified_keys.append(key)
@@ -338,7 +340,7 @@ class LockingFileSystemStore(FileSystemStore):
     ##########################################################################
 
     def _get_lockfile_path(self, key):
-        return os.path.join(self._root, key + '.lock')
+        return os.path.join(self._root, key)
 
     @contextmanager
     def _locking(self, key, recurse=True, shared=False):
@@ -410,11 +412,12 @@ class LockingFileSystemStore(FileSystemStore):
             size = os.stat(log_path).st_size
             with open(log_path, 'rb') as f:
                 first_log = f.readline()
-                f.seek(size/2)
+                f.seek(size//2)
                 f.readline()
                 split_pos = f.tell()
                 f.seek(0)
-                with open(log_path+'.%s'%first_log.split(' ',1)[0], 'wb') as f2:
+                file_name = '{}.{}'.format(log_path, first_log.split(b' ', 1)[0].decode('ascii'))
+                with open(file_name, 'wb') as f2:
                     f2.write(f.read(split_pos))
                 new_text = f.read()
             with open(log_path, 'wb') as f:
@@ -436,7 +439,7 @@ class LockingFileSystemStore(FileSystemStore):
             if last_emit >= self._remote_event_poll_interval:
                 try:
                     last_log = self._check_remote_event(last_log)
-                except OSError as e:
+                except OSError:
                     # Store got deleted
                     return
                 last_emit -= self._remote_event_poll_interval
@@ -451,31 +454,35 @@ class LockingFileSystemStore(FileSystemStore):
             with self._locking(self._log_file, recurse=True):
                 try:
                     f = open(self._log_file)
-                except IOError as e:
+                    size = os.stat(self._log_file).st_size
+                    f.seek(max(size - 1024, 0))
+                    text = f.read(1024)
+                    lines = text.splitlines()
+                    if lines:
+                        id = lines[-1][0]
+                except IOError:
                     return None
-                size = os.stat(self._log_file).st_size
-                f.seek(max(size-1024, 0))
-                text = f.read(1024)
-                lines = text.splitlines()
-                if lines:
-                    id = lines[-1][0]
+                finally:
+                    f.close()
         else:
             with self._locking(self._log_file, recurse=True):
                 try:
                     f = open(self._log_file)
-                except IOError as e:
+                    text = f.read()
+                    seek = self._search_log(id, text)
+                    if seek < 0:
+                        return None
+                    text = text[seek:]
+                    for line in text.splitlines():
+                        try:
+                            id, typ, date, time, key = line.split(' ', 4)
+                            self._emit_remote_event(id, typ, date, time, key)
+                        except ValueError:
+                            pass
+                except IOError:
                     return None
-                text = f.read()
-                seek = self._search_log(id, text)
-                if seek < 0:
-                    return None
-                text = text[seek:]
-                for line in text.splitlines():
-                    try:
-                        id, typ, date, time, key = line.split(' ', 4)
-                        self._emit_remote_event(id, typ, date, time, key)
-                    except ValueError as e:
-                        pass
+                finally:
+                    f.close()
         return id
 
     def _search_log(self, id, text):
@@ -485,7 +492,7 @@ class LockingFileSystemStore(FileSystemStore):
             return 0
         try:
             return text.index('\n'+id)+1
-        except ValueError as e:
+        except ValueError:
             return -1
 
     def _emit_remote_event(self, id, typ, date, time, key):
@@ -510,17 +517,18 @@ class LockingFileSystemStore(FileSystemStore):
         key is the str key which is modified in the commit.
 
         """
+        ISO_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
         try:
-            with open(self._log_file, 'rb') as log_file:
-                ISO_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
+            with io.open(self._log_file, 'r', encoding='utf-8') as log_file:
                 for line in log_file:
                     line = line.rstrip('\n')
                     id, typ, date, time, key = line.split(' ', 4)
-                    yield int(id), typ, datetime.datetime.strptime(date+' '+time,
-                                                         ISO_FORMAT), key
-        except (IOError, ValueError) as e:
+                    tstamp = datetime.datetime.strptime(
+                        '%s %s' % (date, time), ISO_FORMAT
+                    )
+                    yield int(id), typ, tstamp, key
+        except (IOError, ValueError) as exc:
             pass
 
     def __del__(self):
         self._remote_poll_thread = None
-
